#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = __importDefault(require("commander"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const server_1 = require("./server");
function commaSeparatedList(value) {
    return value ? value.split(",") : [];
}
function main(argv) {
    return __awaiter(this, void 0, void 0, function* () {
        commander_1.default
            .option("-m, --mode <mode>", "Mode (record, replay or passthrough)")
            .option("-t, --tapes-dir <tapes-dir>", "Directory in which to record/replay tapes")
            .option("--default-tape <tape-name>", "Name of the default tape", "default")
            .option("-h, --host <host>", "Host to proxy (not required in replay mode)")
            .option("-p, --port <port>", "Local port to serve on", "3000")
            .option("-r, --redact-headers <headers>", "Request headers to redact", commaSeparatedList)
            .parse(argv);
        const initialMode = (commander_1.default.mode || "").toLowerCase();
        const tapeDir = commander_1.default.tapesDir;
        const defaultTapeName = commander_1.default.defaultTape;
        const host = commander_1.default.host;
        const port = parseInt(commander_1.default.port, 10);
        const redactHeaders = commander_1.default.redactHeaders;
        switch (initialMode) {
            case "record":
            case "replay":
            case "mimic":
            case "passthrough":
                // Valid modes.
                break;
            default:
                panic("Please specify a valid mode (record or replay).");
                throw new Error(); // only used for TypeScript control flow
        }
        if (!tapeDir && initialMode !== "passthrough") {
            panic("Please specify a path to a tapes directory.");
        }
        if (initialMode === "replay" && !fs_extra_1.default.existsSync(tapeDir)) {
            panic(`No tapes found at ${tapeDir}. Did you mean to start in record mode?`);
        }
        // Expect a host unless we're in replay mode.
        if (initialMode !== "replay") {
            if (!host) {
                panic("Please specify a host.");
            }
            if (host.indexOf("://") === -1) {
                panic("Please include the scheme (http:// or https://) in the host.");
            }
        }
        const server = new server_1.RecordReplayServer({
            initialMode,
            tapeDir,
            host,
            defaultTapeName,
            enableLogging: true,
            redactHeaders,
        });
        yield server.start(port);
        console.log(chalk_1.default.green(`Proxying in ${initialMode} mode on port ${port}.`));
    });
}
function panic(message) {
    console.error(chalk_1.default.red(message));
    process.exit(1);
}
if (require.main === module) {
    main(process.argv);
}
